@@ -14,6 +14,7 @@
 
 mod util;
 
+use std::convert::TryInto;
 use std::{mem, sync::Arc, thread, time::Duration};
 
 use pretty_assertions::assert_eq;
@@ -21,11 +22,11 @@ use pretty_assertions::assert_eq;
 use rocksdb::{
     perf::get_memory_usage_stats, BlockBasedOptions, BottommostLevelCompaction, Cache,
     ColumnFamilyDescriptor, CompactOptions, CuckooTableOptions, DBAccess, DBCompactionStyle,
-    DBWithThreadMode, Env, Error, FifoCompactOptions, IteratorMode, MultiThreaded, Options,
-    PerfContext, PerfMetric, ReadOptions, SingleThreaded, SliceTransform, Snapshot,
+    DBWithThreadMode, Env, Error, ErrorKind, FifoCompactOptions, IteratorMode, MultiThreaded,
+    Options, PerfContext, PerfMetric, ReadOptions, SingleThreaded, SliceTransform, Snapshot,
     UniversalCompactOptions, UniversalCompactionStopStyle, WriteBatch, DB,
 };
-use util::DBPath;
+use util::{assert_iter, pair, DBPath};
 
 #[test]
 fn external() {
@@ -72,7 +73,7 @@ fn errors_do_stuff() {
     match DB::destroy(&opts, &path) {
         Err(s) => {
             let message = s.to_string();
-            assert!(message.contains("IO error:"));
+            assert_eq!(s.kind(), ErrorKind::IOError);
             assert!(message.contains("_rust_rocksdb_error"));
             assert!(message.contains("/LOCK:"));
         }
@@ -147,7 +148,7 @@ fn iterator_test() {
 
         let iter = db.iterator(IteratorMode::Start);
 
-        for (idx, (db_key, db_value)) in iter.enumerate() {
+        for (idx, (db_key, db_value)) in iter.map(Result::unwrap).enumerate() {
             let (key, value) = data[idx];
             assert_eq!((&key[..], &value[..]), (db_key.as_ref(), db_value.as_ref()));
         }
@@ -188,7 +189,7 @@ fn iterator_test_tailing() {
         }
 
         let mut tot = 0;
-        for (i, (k, v)) in tail_iter.enumerate() {
+        for (i, (k, v)) in tail_iter.map(Result::unwrap).enumerate() {
             assert_eq!(
                 (k.to_vec(), v.to_vec()),
                 (data[i].0.to_vec(), data[i].1.to_vec())
@@ -213,12 +214,10 @@ fn iterator_test_upper_bound() {
         let mut readopts = ReadOptions::default();
         readopts.set_iterate_upper_bound(b"k4".to_vec());
 
-        let iter = db.iterator_opt(IteratorMode::Start, readopts);
-        let expected: Vec<_> = vec![(b"k1", b"v1"), (b"k2", b"v2"), (b"k3", b"v3")]
-            .into_iter()
-            .map(|(k, v)| (k.to_vec().into_boxed_slice(), v.to_vec().into_boxed_slice()))
-            .collect();
-        assert_eq!(expected, iter.collect::<Vec<_>>());
+        assert_iter(
+            db.iterator_opt(IteratorMode::Start, readopts),
+            &[pair(b"k1", b"v1"), pair(b"k2", b"v2"), pair(b"k3", b"v3")],
+        );
     }
 }
 
@@ -236,12 +235,10 @@ fn iterator_test_lower_bound() {
         let mut readopts = ReadOptions::default();
         readopts.set_iterate_lower_bound(b"k4".to_vec());
 
-        let iter = db.iterator_opt(IteratorMode::Start, readopts);
-        let expected: Vec<_> = vec![(b"k4", b"v4"), (b"k5", b"v5")]
-            .into_iter()
-            .map(|(k, v)| (k.to_vec().into_boxed_slice(), v.to_vec().into_boxed_slice()))
-            .collect();
-        assert_eq!(expected, iter.collect::<Vec<_>>());
+        assert_iter(
+            db.iterator_opt(IteratorMode::Start, readopts),
+            &[pair(b"k4", b"v4"), pair(b"k5", b"v5")],
+        );
     }
 }
 
@@ -412,6 +409,39 @@ fn test_get_updates_since_empty() {
     assert!(iter.next().is_none());
 }
 
+#[test]
+fn test_get_updates_since_start() {
+    let path = DBPath::new("_rust_rocksdb_test_test_get_updates_since_start");
+    let db = DB::open_default(&path).unwrap();
+    // add some records and collect sequence numbers,
+    // verify 4 batches of 1 put each were done
+    let seq0 = db.latest_sequence_number();
+    db.put(b"key1", b"value1").unwrap();
+    db.put(b"key2", b"value2").unwrap();
+    db.put(b"key3", b"value3").unwrap();
+    db.put(b"key4", b"value4").unwrap();
+    let mut iter = db.get_updates_since(seq0).unwrap();
+    let mut counts = OperationCounts {
+        puts: 0,
+        deletes: 0,
+    };
+    let (seq, batch) = iter.next().unwrap().unwrap();
+    assert_eq!(seq, 1);
+    batch.iterate(&mut counts);
+    let (seq, batch) = iter.next().unwrap().unwrap();
+    assert_eq!(seq, 2);
+    batch.iterate(&mut counts);
+    let (seq, batch) = iter.next().unwrap().unwrap();
+    assert_eq!(seq, 3);
+    batch.iterate(&mut counts);
+    let (seq, batch) = iter.next().unwrap().unwrap();
+    assert_eq!(seq, 4);
+    batch.iterate(&mut counts);
+    assert!(iter.next().is_none());
+    assert_eq!(counts.puts, 4);
+    assert_eq!(counts.deletes, 0);
+}
+
 #[test]
 fn test_get_updates_since_multiple_batches() {
     let path = DBPath::new("_rust_rocksdb_test_get_updates_since_multiple_batches");
@@ -428,13 +458,13 @@ fn test_get_updates_since_multiple_batches() {
         puts: 0,
         deletes: 0,
     };
-    let (seq, batch) = iter.next().unwrap();
+    let (seq, batch) = iter.next().unwrap().unwrap();
     assert_eq!(seq, 2);
     batch.iterate(&mut counts);
-    let (seq, batch) = iter.next().unwrap();
+    let (seq, batch) = iter.next().unwrap().unwrap();
     assert_eq!(seq, 3);
     batch.iterate(&mut counts);
-    let (seq, batch) = iter.next().unwrap();
+    let (seq, batch) = iter.next().unwrap().unwrap();
     assert_eq!(seq, 4);
     batch.iterate(&mut counts);
     assert!(iter.next().is_none());
@@ -461,7 +491,7 @@ fn test_get_updates_since_one_batch() {
         puts: 0,
         deletes: 0,
     };
-    let (seq, batch) = iter.next().unwrap();
+    let (seq, batch) = iter.next().unwrap().unwrap();
     assert_eq!(seq, 2);
     batch.iterate(&mut counts);
     assert!(iter.next().is_none());
@@ -576,7 +606,7 @@ fn test_open_cf_with_ttl() {
     let mut opts = Options::default();
     opts.create_if_missing(true);
     opts.create_missing_column_families(true);
-    let db = DB::open_cf_with_ttl(&opts, &path, &["test_cf"], Duration::from_secs(1)).unwrap();
+    let db = DB::open_cf_with_ttl(&opts, &path, ["test_cf"], Duration::from_secs(1)).unwrap();
     let cf = db.cf_handle("test_cf").unwrap();
     db.put_cf(&cf, b"key1", b"value1").unwrap();
 
@@ -618,6 +648,23 @@ fn test_open_with_multiple_refs_as_single_threaded() {
     t.compile_fail("tests/fail/open_with_multiple_refs_as_single_threaded.rs");
 }
 
+#[test]
+fn test_open_utf8_path() {
+    let path = DBPath::new("_rust_rocksdb_utf8_path_temporärer_Ordner");
+
+    {
+        let db = DB::open_default(&path).unwrap();
+
+        assert!(db.put(b"k1", b"v1111").is_ok());
+
+        let r: Result<Option<Vec<u8>>, Error> = db.get(b"k1");
+
+        assert_eq!(r.unwrap().unwrap(), b"v1111");
+        assert!(db.delete(b"k1").is_ok());
+        assert!(db.get(b"k1").unwrap().is_none());
+    }
+}
+
 #[test]
 fn compact_range_test() {
     let path = DBPath::new("_rust_rocksdb_compact_range_test");
@@ -697,7 +744,7 @@ fn fifo_compaction_test() {
 
         let block_cache_hit_count = ctx.metric(PerfMetric::BlockCacheHitCount);
         if block_cache_hit_count > 0 {
-            let expect = format!("block_cache_hit_count = {}", block_cache_hit_count);
+            let expect = format!("block_cache_hit_count = {block_cache_hit_count}");
             assert!(ctx.report(true).contains(&expect));
         }
 
@@ -727,7 +774,7 @@ fn env_and_dbpaths_test() {
         opts.create_missing_column_families(true);
 
         {
-            let mut env = Env::default().unwrap();
+            let mut env = Env::new().unwrap();
             env.lower_high_priority_thread_pool_cpu_priority();
             opts.set_env(&env);
         }
@@ -767,12 +814,14 @@ fn prefix_extract_and_iterate_test() {
         readopts.set_iterate_lower_bound(b"p1".to_vec());
         readopts.set_pin_data(true);
 
-        let iter = db.iterator_opt(IteratorMode::Start, readopts);
-        let expected: Vec<_> = vec![(b"p1_k1", b"v1"), (b"p1_k3", b"v3"), (b"p1_k4", b"v4")]
-            .into_iter()
-            .map(|(k, v)| (k.to_vec().into_boxed_slice(), v.to_vec().into_boxed_slice()))
-            .collect();
-        assert_eq!(expected, iter.collect::<Vec<_>>());
+        assert_iter(
+            db.iterator_opt(IteratorMode::Start, readopts),
+            &[
+                pair(b"p1_k1", b"v1"),
+                pair(b"p1_k3", b"v3"),
+                pair(b"p1_k4", b"v4"),
+            ],
+        );
     }
 }
 
@@ -801,7 +850,7 @@ fn get_with_cache_and_bulkload_test() {
 
     {
         // set block based table and cache
-        let cache = Cache::new_lru_cache(512 << 10).unwrap();
+        let cache = Cache::new_lru_cache(512 << 10);
         assert_eq!(cache.get_usage(), 0);
         let mut block_based_opts = BlockBasedOptions::default();
         block_based_opts.set_block_cache(&cache);
@@ -814,7 +863,7 @@ fn get_with_cache_and_bulkload_test() {
         // write a lot
         let mut batch = WriteBatch::default();
         for i in 0..10_000 {
-            batch.put(format!("{:0>4}", i).as_bytes(), b"v");
+            batch.put(format!("{i:0>4}").as_bytes(), b"v");
         }
         assert!(db.write(batch).is_ok());
 
@@ -842,8 +891,8 @@ fn get_with_cache_and_bulkload_test() {
 
         // try to get key
         let iter = db.iterator(IteratorMode::Start);
-        for (expected, (k, _)) in iter.enumerate() {
-            assert_eq!(k.as_ref(), format!("{:0>4}", expected).as_bytes());
+        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
         }
 
         // check live files (sst files meta)
@@ -894,7 +943,7 @@ fn get_with_cache_and_bulkload_test() {
         opts.set_stats_persist_period_sec(0);
 
         // test Env::Default()->SetBackgroundThreads(0, Env::Priority::BOTTOM);
-        let mut env = Env::default().unwrap();
+        let mut env = Env::new().unwrap();
         env.set_bottom_priority_background_threads(0);
         opts.set_env(&env);
 
@@ -903,8 +952,143 @@ fn get_with_cache_and_bulkload_test() {
 
         // try to get key
         let iter = db.iterator(IteratorMode::Start);
-        for (expected, (k, _)) in iter.enumerate() {
-            assert_eq!(k.as_ref(), format!("{:0>4}", expected).as_bytes());
+        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
+        }
+    }
+}
+
+#[test]
+fn get_with_cache_and_bulkload_and_blobs_test() {
+    let path = DBPath::new("_rust_rocksdb_get_with_cache_and_bulkload_and_blobs_test");
+    let log_path = DBPath::new("_rust_rocksdb_log_path_test");
+
+    // create options
+    let mut opts = Options::default();
+    opts.create_if_missing(true);
+    opts.create_missing_column_families(true);
+    opts.set_wal_bytes_per_sync(8 << 10); // 8KB
+    opts.set_writable_file_max_buffer_size(512 << 10); // 512KB
+    opts.set_enable_write_thread_adaptive_yield(true);
+    opts.set_unordered_write(true);
+    opts.set_max_subcompactions(2);
+    opts.set_max_background_jobs(4);
+    opts.set_use_adaptive_mutex(true);
+    opts.set_db_log_dir(&log_path);
+    opts.set_memtable_whole_key_filtering(true);
+    opts.set_dump_malloc_stats(true);
+    opts.set_enable_blob_files(true);
+    opts.set_min_blob_size(256); // set small to ensure it is actually used
+
+    // trigger all sst files in L1/2 instead of L0
+    opts.set_max_bytes_for_level_base(64 << 10); // 64KB
+
+    {
+        // set block based table and cache
+        let cache = Cache::new_lru_cache(512 << 10);
+        assert_eq!(cache.get_usage(), 0);
+        let mut block_based_opts = BlockBasedOptions::default();
+        block_based_opts.set_block_cache(&cache);
+        block_based_opts.set_cache_index_and_filter_blocks(true);
+        opts.set_block_based_table_factory(&block_based_opts);
+
+        // open db
+        let db = DB::open(&opts, &path).unwrap();
+
+        // write a lot
+        let mut batch = WriteBatch::default();
+        for i in 0..10_000 {
+            batch.put(format!("{i:0>4}").as_bytes(), b"v");
+        }
+        assert!(db.write(batch).is_ok());
+
+        // flush memory table to sst and manual compaction
+        assert!(db.flush().is_ok());
+        db.compact_range(Some(format!("{:0>4}", 0).as_bytes()), None::<Vec<u8>>);
+
+        // get -> trigger caching
+        let _ = db.get(b"1");
+        assert!(cache.get_usage() > 0);
+
+        // get approximated memory usage
+        let mem_usage = get_memory_usage_stats(Some(&[&db]), None).unwrap();
+        assert!(mem_usage.mem_table_total > 0);
+
+        // get approximated cache usage
+        let mem_usage = get_memory_usage_stats(None, Some(&[&cache])).unwrap();
+        assert!(mem_usage.cache_total > 0);
+    }
+
+    // bulk loading
+    {
+        // open db
+        let db = DB::open(&opts, &path).unwrap();
+
+        // try to get key
+        let iter = db.iterator(IteratorMode::Start);
+        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
+        }
+
+        // check live files (sst files meta)
+        let livefiles = db.live_files().unwrap();
+        assert_eq!(livefiles.len(), 1);
+        livefiles.iter().for_each(|f| {
+            assert_eq!(f.level, 2);
+            assert_eq!(f.column_family_name, "default");
+            assert!(!f.name.is_empty());
+            assert_eq!(
+                f.start_key.as_ref().unwrap().as_slice(),
+                format!("{:0>4}", 0).as_bytes()
+            );
+            assert_eq!(
+                f.end_key.as_ref().unwrap().as_slice(),
+                format!("{:0>4}", 9999).as_bytes()
+            );
+            assert_eq!(f.num_entries, 10000);
+            assert_eq!(f.num_deletions, 0);
+        });
+
+        // delete sst file in range (except L0)
+        assert!(db
+            .delete_file_in_range(
+                format!("{:0>4}", 0).as_bytes(),
+                format!("{:0>4}", 9999).as_bytes()
+            )
+            .is_ok());
+        let livefiles = db.live_files().unwrap();
+        assert_eq!(livefiles.len(), 0);
+
+        // try to get a deleted key
+        assert!(db.get(format!("{:0>4}", 123).as_bytes()).unwrap().is_none());
+    }
+
+    // raise error when db exists
+    {
+        opts.set_error_if_exists(true);
+        assert!(DB::open(&opts, &path).is_err());
+    }
+
+    // disable all threads
+    {
+        // create new options
+        let mut opts = Options::default();
+        opts.set_max_background_jobs(0);
+        opts.set_stats_dump_period_sec(0);
+        opts.set_stats_persist_period_sec(0);
+
+        // test Env::Default()->SetBackgroundThreads(0, Env::Priority::BOTTOM);
+        let mut env = Env::new().unwrap();
+        env.set_bottom_priority_background_threads(0);
+        opts.set_env(&env);
+
+        // open db
+        let db = DB::open(&opts, &path).unwrap();
+
+        // try to get key
+        let iter = db.iterator(IteratorMode::Start);
+        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
         }
     }
 }
@@ -1011,7 +1195,7 @@ fn multi_get() {
         let k1_snap = db.snapshot();
         db.put(b"k2", b"v2").unwrap();
 
-        let _ = db.multi_get(&[b"k0"; 40]);
+        let _ = db.multi_get([b"k0"; 40]);
 
         let assert_values = |values: Vec<_>| {
             assert_eq!(3, values.len());
@@ -1021,14 +1205,14 @@ fn multi_get() {
         };
 
         let values = db
-            .multi_get(&[b"k0", b"k1", b"k2"])
+            .multi_get([b"k0", b"k1", b"k2"])
             .into_iter()
             .map(Result::unwrap)
             .collect::<Vec<_>>();
 
         assert_values(values);
 
-        let values = DBAccess::multi_get_opt(&db, &[b"k0", b"k1", b"k2"], &Default::default())
+        let values = DBAccess::multi_get_opt(&db, [b"k0", b"k1", b"k2"], &Default::default())
             .into_iter()
             .map(Result::unwrap)
             .collect::<Vec<_>>();
@@ -1037,7 +1221,7 @@ fn multi_get() {
 
         let values = db
             .snapshot()
-            .multi_get(&[b"k0", b"k1", b"k2"])
+            .multi_get([b"k0", b"k1", b"k2"])
             .into_iter()
             .map(Result::unwrap)
             .collect::<Vec<_>>();
@@ -1045,7 +1229,7 @@ fn multi_get() {
         assert_values(values);
 
         let none_values = initial_snap
-            .multi_get(&[b"k0", b"k1", b"k2"])
+            .multi_get([b"k0", b"k1", b"k2"])
             .into_iter()
             .map(Result::unwrap)
             .collect::<Vec<_>>();
@@ -1053,7 +1237,7 @@ fn multi_get() {
         assert_eq!(none_values, vec![None; 3]);
 
         let k1_only = k1_snap
-            .multi_get(&[b"k0", b"k1", b"k2"])
+            .multi_get([b"k0", b"k1", b"k2"])
             .into_iter()
             .map(Result::unwrap)
             .collect::<Vec<_>>();
@@ -1070,7 +1254,7 @@ fn multi_get_cf() {
         let mut opts = Options::default();
         opts.create_if_missing(true);
         opts.create_missing_column_families(true);
-        let db = DB::open_cf(&opts, &path, &["cf0", "cf1", "cf2"]).unwrap();
+        let db = DB::open_cf(&opts, &path, ["cf0", "cf1", "cf2"]).unwrap();
 
         let cf0 = db.cf_handle("cf0").unwrap();
 
@@ -1092,6 +1276,33 @@ fn multi_get_cf() {
     }
 }
 
+#[test]
+fn batched_multi_get_cf() {
+    let path = DBPath::new("_rust_rocksdb_batched_multi_get_cf");
+
+    {
+        let mut opts = Options::default();
+        opts.create_if_missing(true);
+        opts.create_missing_column_families(true);
+        let db = DB::open_cf(&opts, &path, ["cf0"]).unwrap();
+
+        let cf = db.cf_handle("cf0").unwrap();
+        db.put_cf(&cf, b"k1", b"v1").unwrap();
+        db.put_cf(&cf, b"k2", b"v2").unwrap();
+
+        let values = db
+            .batched_multi_get_cf(&cf, vec![b"k0", b"k1", b"k2"], true) // sorted_input
+            .into_iter()
+            .map(Result::unwrap)
+            .collect::<Vec<_>>();
+        assert_eq!(3, values.len());
+        assert!(values[0].is_none());
+        assert!(values[1].is_some());
+        assert_eq!(&(values[1].as_ref().unwrap())[0..2], b"v1");
+        assert_eq!(&(values[2].as_ref().unwrap())[0..2], b"v2");
+    }
+}
+
 #[test]
 fn key_may_exist() {
     let path = DBPath::new("_rust_key_may_exist");
@@ -1111,7 +1322,7 @@ fn key_may_exist_cf() {
         let mut opts = Options::default();
         opts.create_if_missing(true);
         opts.create_missing_column_families(true);
-        let db = DB::open_cf(&opts, &path, &["cf"]).unwrap();
+        let db = DB::open_cf(&opts, &path, ["cf"]).unwrap();
         let cf = db.cf_handle("cf").unwrap();
 
         assert!(!db.key_may_exist_cf(&cf, "nonexistent"));
@@ -1119,6 +1330,38 @@ fn key_may_exist_cf() {
     }
 }
 
+#[test]
+fn key_may_exist_cf_value() {
+    let path = DBPath::new("_rust_key_may_exist_cf_value");
+
+    {
+        let mut opts = Options::default();
+        opts.create_if_missing(true);
+        opts.create_missing_column_families(true);
+        let db = DB::open_cf(&opts, &path, ["cf"]).unwrap();
+        let cf = db.cf_handle("cf").unwrap();
+
+        // put some entry into db
+        for i in 0..10000i32 {
+            let _ = db.put_cf(&cf, i.to_le_bytes(), i.to_le_bytes());
+        }
+
+        // call `key_may_exist_cf_opt_value`
+        for i in 0..10000i32 {
+            let (may_exist, value) =
+                db.key_may_exist_cf_opt_value(&cf, i.to_le_bytes(), &ReadOptions::default());
+
+            // all these numbers may exist
+            assert!(may_exist);
+
+            // check value correctness
+            if let Some(value) = value {
+                assert_eq!(i32::from_le_bytes(value.as_ref().try_into().unwrap()), i);
+            }
+        }
+    }
+}
+
 #[test]
 fn test_snapshot_outlive_db() {
     let t = trybuild::TestCases::new();
